@@ -324,6 +324,8 @@ struct update_t {
 		memcpy(hash, from.hash, sizeof(hash));
 		memcpy(downloadhash, from.downloadhash, sizeof(downloadhash));
 		memcpy(my_hash, from.my_hash, sizeof(my_hash));
+
+		return *this;
 	}
 };
 
@@ -356,11 +358,11 @@ bool DownloadWorkerThread()
 	WinHttpSetOption(hSession, WINHTTP_OPTION_SECURE_PROTOCOLS,
 		(LPVOID)&tlsProtocols, sizeof(tlsProtocols));
 
-	HttpHandle hConnect = WinHttpConnect(hSession, L"obsproject.com",
+	HttpHandle hConnect = WinHttpConnect(hSession, L"cdn-fastly.obsproject.com",
 			INTERNET_DEFAULT_HTTPS_PORT, 0);
 	if (!hConnect) {
 		downloadThreadFailure = true;
-		Status(L"Update failed: Couldn't connect to obsproject.com");
+		Status(L"Update failed: Couldn't connect to cdn-fastly.obsproject.com");
 		return false;
 	}
 
@@ -574,10 +576,18 @@ static inline bool FileExists(const wchar_t *path)
 
 static bool NonCorePackageInstalled(const char *name)
 {
-	if (strcmp(name, "obs-browser") == 0) {
-		return FileExists(L"obs-plugins\\32bit\\obs-browser.dll");
-	} else if (strcmp(name, "realsense") == 0) {
-		return FileExists(L"obs-plugins\\32bit\\win-ivcam.dll");
+	if (is32bit) {
+		if (strcmp(name, "obs-browser") == 0) {
+			return FileExists(L"obs-plugins\\32bit\\obs-browser.dll");
+		} else if (strcmp(name, "realsense") == 0) {
+			return FileExists(L"obs-plugins\\32bit\\win-ivcam.dll");
+		}
+	} else {
+		if (strcmp(name, "obs-browser") == 0) {
+			return FileExists(L"obs-plugins\\64bit\\obs-browser.dll");
+		} else if (strcmp(name, "realsense") == 0) {
+			return FileExists(L"obs-plugins\\64bit\\win-ivcam.dll");
+		}
 	}
 
 	return false;
@@ -612,7 +622,7 @@ static inline bool has_str(const char *file, const char *str)
 #define UTF8ToWideBuf(wide, utf8) UTF8ToWide(wide, _countof(wide), utf8)
 #define WideToUTF8Buf(utf8, wide) WideToUTF8(utf8, _countof(utf8), wide)
 
-#define UPDATE_URL L"https://obsproject.com/update_studio"
+#define UPDATE_URL L"https://cdn-fastly.obsproject.com/update_studio"
 
 static bool AddPackageUpdateFiles(json_t *root, size_t idx,
 		const wchar_t *tempPath)
@@ -748,7 +758,7 @@ static void UpdateWithPatchIfAvailable(const char *name, const char *hash,
 	wchar_t sourceURL[1024];
 	wchar_t patchHashStr[BLAKE2_HASH_STR_LENGTH];
 
-	if (strncmp(source, "https://obsproject.com/", 23) != 0)
+	if (strncmp(source, "https://cdn-fastly.obsproject.com/", 34) != 0)
 		return;
 
 	string patchPackageName = name;
@@ -957,10 +967,10 @@ static bool UpdateVS2017Redists(json_t *root)
 	WinHttpSetOption(hSession, WINHTTP_OPTION_SECURE_PROTOCOLS,
 		(LPVOID)&tlsProtocols, sizeof(tlsProtocols));
 
-	HttpHandle hConnect = WinHttpConnect(hSession, L"obsproject.com",
+	HttpHandle hConnect = WinHttpConnect(hSession, L"cdn-fastly.obsproject.com",
 			INTERNET_DEFAULT_HTTPS_PORT, 0);
 	if (!hConnect) {
-		Status(L"Update failed: Couldn't connect to obsproject.com");
+		Status(L"Update failed: Couldn't connect to cdn-fastly.obsproject.com");
 		return false;
 	}
 
@@ -981,7 +991,7 @@ static bool UpdateVS2017Redists(json_t *root)
 		: L"vc2017redist_x64.exe";
 
 	wstring sourceURL;
-	sourceURL += L"https://obsproject.com/downloads/";
+	sourceURL += L"https://cdn-fastly.obsproject.com/downloads/";
 	sourceURL += file;
 
 	wstring destPath;
