@@ -645,13 +645,7 @@ static inline bool update_frame(av_capture *capture,
 
 static void av_capture_enable_buffering(av_capture *capture, bool enabled)
 {
-	obs_source_t *source = capture->source;
-	uint32_t flags = obs_source_get_flags(source);
-	if (enabled)
-		flags &= ~OBS_SOURCE_FLAG_UNBUFFERED;
-	else
-		flags |= OBS_SOURCE_FLAG_UNBUFFERED;
-	obs_source_set_flags(source, flags);
+	obs_source_set_async_unbuffered(capture->source, !enabled);
 }
 
 static const char *av_capture_getname(void*)
@@ -2181,6 +2175,10 @@ static void av_capture_update(void *data, obs_data_t *settings)
 
 OBS_DECLARE_MODULE()
 OBS_MODULE_USE_DEFAULT_LOCALE("mac-avcapture", "en-US")
+MODULE_EXPORT const char *obs_module_description(void)
+{
+	return "MacOS AVFoundation Capture source";
+}
 
 bool obs_module_load(void)
 {
