@@ -1,4 +1,3 @@
-#define _CRT_SECURE_NO_WARNINGS
 #include <inttypes.h>
 #include <stdio.h>
 #include <windows.h>
@@ -16,6 +15,8 @@ int main(int argc, char *argv[])
 	wc.lpfnWndProc   = (WNDPROC)DefWindowProcA;
 	wc.lpszClassName = DUMMY_WNDCLASS;
 
+	SetErrorMode(SEM_FAILCRITICALERRORS);
+
 	if (!RegisterClassA(&wc)) {
 		printf("failed to register '%s'\n", DUMMY_WNDCLASS);
 		return -1;
@@ -27,15 +28,15 @@ int main(int argc, char *argv[])
 
 	printf("[d3d8]\n");
 	printf("present=0x%"PRIx32"\n", d3d8.present);
-	printf("reset=0x%"PRIx32"\n", d3d8.reset);
 	printf("[d3d9]\n");
 	printf("present=0x%"PRIx32"\n", d3d9.present);
 	printf("present_ex=0x%"PRIx32"\n", d3d9.present_ex);
 	printf("present_swap=0x%"PRIx32"\n", d3d9.present_swap);
-	printf("reset=0x%"PRIx32"\n", d3d9.reset);
-	printf("reset_ex=0x%"PRIx32"\n", d3d9.reset_ex);
+	printf("d3d9_clsoff=0x%"PRIx32"\n", d3d9.d3d9_clsoff);
+	printf("is_d3d9ex_clsoff=0x%"PRIx32"\n", d3d9.is_d3d9ex_clsoff);
 	printf("[dxgi]\n");
 	printf("present=0x%"PRIx32"\n", dxgi.present);
+	printf("present1=0x%"PRIx32"\n", dxgi.present1);
 	printf("resize=0x%"PRIx32"\n", dxgi.resize);
 
 	(void)argc;
